/*
 *  
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * TODO :
 *  - deal with multiple open (VIDIOC_S_FMT, priority, io, ...)
 *  - tune of encoder settings.
 *  - Radio tuner.
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/smp_lock.h>
#include <linux/completion.h>
#include <asm/uaccess.h>
#include <linux/usb.h>

#include <linux/slab.h>
#include <linux/usb.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/smp_lock.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/signal.h>
#include <linux/poll.h>
#include <linux/proc_fs.h>

#include <linux/videodev.h>
#include <linux/firmware.h>

#include "pvrusb2.h"
#include "pvrusb2-data.h"
#include "pvrusb2-v4l2.h"
#include "pvrusb2-remote.h"

#define DRIVER_VERSION "0.5(10/08/2004)"
#define DRIVER_AUTHOR "Aurelien Alleaume <slts@free.fr>"
#define DRIVER_DESC "Hauppauge WinTV-PVR-USB2 MPEG2 Encoder/Tuner"

static int video_nr = -1;       // Number of Video Devices Initialized
static int pvr_nr = 0;          // sequential number of pvrusb2 device

module_param(video_nr, int, -1);
MODULE_PARM_DESC(video_nr, "Offset for the minor of the devices");

static struct usb_device_id pvr_table[] = {
	{ USB_DEVICE(0x2040, 0x2900) },
	{ }
};

static void pvr_state_change(struct pvr *, enum pvr_state);
static inline void pvr_state_above(struct pvr *, enum pvr_state);
static inline void pvr_state_below(struct pvr *, enum pvr_state);

/* 
 * pvr_reset_ctl_endpoints().
 *
 * Must be used before sending or receiving data via the bulk endpoints used
 * to transmit commands.
 *
 */

static void pvr_reset_ctl_endpoints(struct pvr *pvr)
{
    usb_settoggle(pvr->udev, PVR_CTL_WRITE_ENDPOINT & 0xf,
                  !(PVR_CTL_WRITE_ENDPOINT & USB_DIR_IN), 0);
    usb_settoggle(pvr->udev, PVR_CTL_READ_ENDPOINT & 0xf,
                  !(PVR_CTL_READ_ENDPOINT & USB_DIR_IN), 0);
    usb_clear_halt(pvr->udev, usb_rcvbulkpipe(pvr->udev,
                                             PVR_CTL_READ_ENDPOINT & 0x7f));
    usb_clear_halt(pvr->udev, usb_sndbulkpipe(pvr->udev,
                                             PVR_CTL_WRITE_ENDPOINT & 0x7f));
}

/* 
 * pvr_send_request().
 *
 * Can be used to send or receive data or both. ctl_sem must be held.
 *
 */

static int pvr_send_request(struct pvr *pvr, void *data,
                            unsigned int dlen, void *res,
                            unsigned int rlen)
{
    int ret = 0, actual_dlen;
    unsigned int pipe;

//    pvr_debug("pvr_send_request : %i", dlen);

    if (pvr->disconnecting || pvr->state == _unstable)
        return -EIO;

    /* send the data if any. */
    if(dlen && data){
        pipe = usb_sndbulkpipe(pvr->udev, PVR_CTL_WRITE_ENDPOINT);
        ret = usb_bulk_msg(pvr->udev, pipe, data, dlen, &actual_dlen, HZ / 4);
        if (ret){
            info("device unstable, trying to reset it"); 
            pvr_state_change(pvr, _need_firmware);
            return ret;
        }
    }

    if(rlen == 0)
        return 0;

    pipe = usb_rcvbulkpipe(pvr->udev, PVR_CTL_READ_ENDPOINT);
    ret = usb_bulk_msg(pvr->udev, pipe, res, rlen, &actual_dlen, HZ / 4);

    if (ret){
        info("device unstable, trying to reset it"); 
        pvr_state_change(pvr, _need_firmware);
        return ret;
    }

    return ret;
}

/* 
 * pvr_send_sequence().
 *
 * executes a sequence of commands ended by a zero length command.
 *
 */

static int pvr_send_sequence(struct pvr *pvr, struct pvr_cmd *cmd)
{
    int ret = 0, i;

//    pvr_debug("pvr_send_sequence");

    i = 0;
    while ((cmd[i].length != 0) && (ret == 0)){
        memcpy(pvr->cmd_buffer, cmd[i].cmd, cmd[i].length);
        ret |= pvr_send_request(pvr, pvr->cmd_buffer, cmd[i].length,
                                pvr->cmd_buffer, cmd[i].ret);
        i++;
    }
    return ret;
}

/* 
 * pvr_upload_firmware1().
 *
 * It will upload the firmware to the inboard USB controller (FX2) ; Making
 * it re-enumerate.
 *
 * NOTE : the pointer to the firmware data given by request_firmware()
 * is not suitable for an usb transaction.
 *
 */

static int pvr_upload_firmware1(struct usb_device *udev, char *firmware)
{
    const struct firmware *fw_entry = 0;
    void  *fw_ptr;
    unsigned int pipe;
    int ret;
    u16 address;

    pvr_debug("pvr_upload_firmware1");

    usb_settoggle(udev, 0 & 0xf, !(0 & USB_DIR_IN), 0);
    usb_clear_halt(udev, usb_sndbulkpipe(udev, 0 & 0x7f));

    pipe = usb_sndctrlpipe(udev, 0);
    ret = request_firmware(&fw_entry, firmware, &udev->dev);
    
    if (ret) {
        err("request_firmware failed for '%s'", firmware);
        return ret;
    }

    if (fw_entry->size != 0x2000){
        err("wrong fx2 firmware size");
        release_firmware(fw_entry);
        return -ENOMEM;
    }

    fw_ptr = kmalloc(0x800, GFP_KERNEL);
    if (fw_ptr == NULL){
        release_firmware(fw_entry);
        return -ENOMEM;
    }

    /*
     * Write 0x01 to the CPUCS register at address e600 to stop the CPU 
     * during firmware upload.
     */
    
    ((u8*) fw_ptr)[0] = 0x01;
    ret = usb_control_msg(udev, pipe, 0xa0, 0x40, 0xe600, 0, fw_ptr, 1, HZ);
    pvr_debug("CPUCS register reset command : %d byte written", ret);
    
    /*
     * upload the firmware to address 0000-1fff in 2048 (=0x800) bytes chunk.
     */

    for(address = 0; address < fw_entry->size; address += 0x800){ 
        memcpy(fw_ptr, fw_entry->data + address, 0x800);
        ret += usb_control_msg(udev, pipe, 0xa0, 0x40, address,
                               0, fw_ptr, 0x800, HZ);
        pvr_debug("Firmware upload : %d bytes written", ret);
    }

    /*
     * Release the CPU from the reset state by writing 0x00 to CPUCS.
     */
    ((char*) fw_ptr)[0] = 0x00;
    ret = usb_control_msg(udev, pipe, 0xa0, 0x40, 0xe600, 0, fw_ptr, 1, HZ);
    pvr_debug("CPUCS register restart command : %d byte written", ret);

    kfree(fw_ptr);
    release_firmware(fw_entry);

    return 0;
}

/* 
 * pvr_buff_complete()
 *
 * This is called after an urb used for video stream is back.
 *
 */

static void pvr_buff_complete (struct urb *purb, struct pt_regs *regs)
{
	struct pvr_buffer *b = (struct pvr_buffer *) purb->context;
	struct pvr *pvr = b->pvr;
	void *buff = purb->transfer_buffer;
    unsigned long flags;

//    pvr_debug("pvr_buff_complete");
    
	if (purb->status != -ENOENT) {
        /* urb was not canceled. Put it in our rx ring. */
        b->data = buff;
        b->size = purb->actual_length;
        spin_lock_irqsave(&pvr->ring_lock, flags);
        pvr->rx_buffer[pvr->rx2 % PVR_NBUFF] = b;
        pvr->rx2++;
        spin_unlock_irqrestore(&pvr->ring_lock, flags);
        if (waitqueue_active(&pvr->wait_data))
		    wake_up (&pvr->wait_data);
    }
    if (atomic_dec_and_test(&pvr->running_urbs)){
        pvr_debug("pvr_buff_complete : no more urbs submitted !");
        wake_up(&pvr->urbs_collected);
    }
}

static int pvr_alloc_video_buff (struct pvr *pvr)
{
    int i, pipe;
    struct pvr_buffer *b;
    void *buff;

    pvr_debug("pvr_alloc_video_buff");

    memset(pvr->buffer, 0, sizeof(struct pvr_buffer)*PVR_NBUFF);
    for (i = 0; i < PVR_NBUFF; i++){
        b = &pvr->buffer[i];
        b->purb = usb_alloc_urb(0, GFP_KERNEL);
        if (b->purb == NULL)
            return -ENOMEM;
        buff = kmalloc(PVR_VID_BUFFSIZE, GFP_KERNEL);
        if (buff == NULL){
            b->purb->transfer_buffer = NULL;
            return -ENOMEM;
        }
        b->pvr = pvr;
        pipe = usb_rcvbulkpipe(pvr->udev, PVR_VID_ENDPOINT);
        usb_fill_bulk_urb (b->purb, pvr->udev, pipe, buff,
                           PVR_VID_BUFFSIZE, pvr_buff_complete, b);
      	b->purb->transfer_flags = 0;
    }

    return 0;
}

static void pvr_free_video_buff (struct pvr *pvr)
{
    int i;
    struct pvr_buffer *b;
    void *ptr;

    pvr_debug("pvr_free_video_buff");
    for (i = 0; i < PVR_NBUFF; i++){
        b = &pvr->buffer[i];
        if (b->purb){
            ptr = b->purb->transfer_buffer;
            usb_free_urb(b->purb);
            b->purb = NULL;
            if (ptr)
                kfree(ptr);
        }
    }
}

#define PVR_DECOMPOSE_LE(t,i,d)   (t)[i] = (d) & 0xff;\
                                  (t)[i+1] = ((d) >> 8) & 0xff;\
                                  (t)[i+2] = ((d) >> 16) & 0xff;\
                                  (t)[i+3] = ((d) >> 24) & 0xff

#define PVR_DECOMPOSE_BE(t,i,d)   (t)[i+3] = (d) & 0xff;\
                                  (t)[i+2] = ((d) >> 8) & 0xff;\
                                  (t)[i+1] = ((d) >> 16) & 0xff;\
                                  (t)[i] = ((d) >> 24) & 0xff


static int pvr_write_register(struct pvr *pvr, u16 reg, u32 data)
{
    int ret;

    pvr_debug("pvr_write_register");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = 0x04;  /* write register prefix */
    PVR_DECOMPOSE_BE(pvr->cmd_buffer,1,data);
    pvr->cmd_buffer[5] = 0;
    pvr->cmd_buffer[6] = (reg >> 8) & 0xff;
    pvr->cmd_buffer[7] = reg & 0xff;


    ret = pvr_send_request(pvr, pvr->cmd_buffer, 8, pvr->cmd_buffer, 0);

    up(&pvr->ctl_sem);

    return ret;
}

static int pvr_read_register(struct pvr *pvr, u16 reg, u32 *data)
{
    int ret = 0;

    pvr_debug("pvr_read_register");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = 0x05;  /* read register prefix */
    pvr->cmd_buffer[1] = 0;
    pvr->cmd_buffer[2] = 0;
    pvr->cmd_buffer[3] = 0;
    pvr->cmd_buffer[4] = 0;
    pvr->cmd_buffer[5] = 0;
    pvr->cmd_buffer[6] = (reg >> 8) & 0xff;
    pvr->cmd_buffer[7] = reg & 0xff;

    ret |= pvr_send_request(pvr, pvr->cmd_buffer, 8, pvr->cmd_buffer, 4);
    *data = *((u32 *)pvr->cmd_buffer) & 0xffff;

    up(&pvr->ctl_sem);

    return ret;
}

static int pvr_write_u16(struct pvr *pvr, u16 data, int res)
{
    int ret;

    pvr_debug("pvr_write_u16");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = (data >> 8) & 0xff;
    pvr->cmd_buffer[1] = data & 0xff;

    ret = pvr_send_request(pvr, pvr->cmd_buffer, 2, pvr->cmd_buffer, res);

    up(&pvr->ctl_sem);

    return ret;
}

static int pvr_write_u8(struct pvr *pvr, u8 data, int res)
{
    int ret;

    pvr_debug("pvr_write_u8");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = data;

    ret = pvr_send_request(pvr, pvr->cmd_buffer, 1, pvr->cmd_buffer, res);

    up(&pvr->ctl_sem);

    return ret;
}

int pvr_read_remote(struct pvr *pvr, int *key)
{
    int ret;

    down(&pvr->ctl_sem);
    
    memset(pvr->cmd_buffer, 0, 0x0c);
    pvr->cmd_buffer[0] = 0x09;  /* read prefix */
    pvr->cmd_buffer[2] = 0x02;
    pvr->cmd_buffer[3] = 0x18;  /* i2c addr of IR receiver */
    
    *key = 0;
    ret  = pvr_send_request(pvr, pvr->cmd_buffer, 0x0c, pvr->cmd_buffer, 4);
    if (ret == 0)
        *key = (pvr->cmd_buffer[1] << 8) + pvr->cmd_buffer[2] ;
    up(&pvr->ctl_sem);

    return ret;
}


/* 
 * pvr_upload_firmware2()
 *
 * This uploads encoder firmware on endpoint 2.
 *
 */

static int pvr_upload_firmware2(struct pvr *pvr)
{
    const struct firmware *fw_entry = 0;
    void  *fw_ptr;
    unsigned int pipe, fw_len, fw_done;
    int actual_length;
    int ret = 0;

    pvr_reset_ctl_endpoints(pvr);
   
    /* First prepare firmware loading */
    ret |= pvr_write_u8(pvr, 0x0b, 1);
    ret |= pvr_write_u16(pvr, 0xde00, 0);
    ret |= pvr_write_u8(pvr, 0xeb, 1);
    ret |= pvr_write_register(pvr, 0x0048, 0xffffffff); /* interrupt mask */
    ret |= pvr_write_register(pvr, 0x9020, 0x88000000); /* gpio dir */
    ret |= pvr_write_register(pvr, 0x900c, 0x08000000); /* gpio output state */
    ret |= pvr_write_u8(pvr, 0xdd, 0);
    ret |= pvr_write_register(pvr, 0xa064, 0x00000000); /* APU command */
    ret |= pvr_write_register(pvr, 0x9020, 0x08040000); /* gpio dir */
    ret |= pvr_write_register(pvr, 0x900c, 0x08000000); /* gpio output state */
    ret |= pvr_write_register(pvr, 0x9058, 0xedffffff); /* VPU ctrl */
    ret |= pvr_write_register(pvr, 0x9054, 0xfdffffff); /* reset hw blocks */
    ret |= pvr_write_register(pvr, 0x07f8, 0x00080080); /* encoder SDRAM refresh */
    ret |= pvr_write_register(pvr, 0x07fc, 0x1a000000); /* encoder SDRAM pre-charge */
    ret |= pvr_write_register(pvr, 0x0700, 0x00000000); /* I2C clock */
    ret |= pvr_write_register(pvr, 0xaa00, 0x00000000); /* unknown */
    ret |= pvr_write_register(pvr, 0xaa04, 0x10780500); /* unknown */
    ret |= pvr_write_register(pvr, 0xaa10, 0x00851400); /* unknown */
    ret |= pvr_write_register(pvr, 0xaa18, 0x00008400); /* unknown */
    ret |= pvr_write_u8(pvr, 0x52, 0);

    if (ret)
        return ret;

    /* Now send firmware */
    
    ret = request_firmware(&fw_entry, FIRMWARE2_FILE, &pvr->udev->dev);

    if (ret) {
        err("request_firmware failed for '%s'", FIRMWARE2_FILE);
        return ret;
    }

    fw_len = fw_entry->size;

    if (fw_len % FIRMWARE_CHUNK_SIZE) {
        err("size of %s must be a multiple of 8192B", FIRMWARE2_FILE);
        release_firmware(fw_entry);
        return -1;
    }

    fw_ptr = kmalloc(FIRMWARE_CHUNK_SIZE, GFP_KERNEL);
    if (fw_ptr == NULL){
        release_firmware(fw_entry);
        return -ENOMEM;
    }

    pipe = usb_sndbulkpipe(pvr->udev, PVR_FIRMWARE_ENDPOINT);

    for (fw_done = 0 ; (fw_done < fw_len) && !ret ; fw_done += FIRMWARE_CHUNK_SIZE ) {
        int i;
        memcpy(fw_ptr, fw_entry->data + fw_done, FIRMWARE_CHUNK_SIZE);
        /* Usbsnoop log  shows that we must swap bytes... */
        for (i = 0; i < FIRMWARE_CHUNK_SIZE/4 ; i++)
           ((u32 *)fw_ptr)[i] = ___swab32(((u32 *)fw_ptr)[i]);

        ret |= usb_bulk_msg(pvr->udev, pipe, fw_ptr, FIRMWARE_CHUNK_SIZE,
                            &actual_length, HZ);
        ret |= (actual_length != FIRMWARE_CHUNK_SIZE);
    }

    pvr_debug("upload of %s : %i / %i ie %i ", FIRMWARE2_FILE, fw_done,
              fw_entry->size, ret);

    kfree(fw_ptr);
    release_firmware(fw_entry);

    if (ret)
        return ret;

    /* Finish upload */

    ret |= pvr_write_register(pvr, 0x9054, 0xffffffff); /* reset hw blocks */
    ret |= pvr_write_register(pvr, 0x9058, 0xe8ffffff); /* VPU ctrl */
    ret |= pvr_write_u16(pvr, 0x0600, 0);

    return ret;
}

/*
 * pvr_do_command()
 *
 * Executes a simple command.
 */

static int pvr_do_command(struct pvr *pvr, struct pvr_cmd *cmd, int lock)
{
    int ret;

    pvr_debug("pvr_do_command");

    if (cmd == NULL) 
        return -EIO;

    if (lock)
        down(&pvr->ctl_sem);

    /* data command (cmd->cmd) is (often) statically allocated. For usb
     *  transfer we need dynamic memory.
    */
    memcpy(pvr->cmd_buffer, cmd->cmd, cmd->length);

    ret = pvr_send_request(pvr, pvr->cmd_buffer, cmd->length,
                           pvr->cmd_buffer, cmd->ret);

    if (lock)
        up(&pvr->ctl_sem);

    return ret;
}



/*
 * pvr_set_freq()
 *
 * Set the frequency to freq (unit is 62.5 KHz).
 *
 */

int pvr_set_freq(struct pvr *pvr)
{
    int ret;
    u32 val;

    pvr_debug("pvr_set_freq");

    down(&pvr->ctl_sem);

    /* Not sure if it's OK for all bands. */
    val = 622 + pvr->frequency;

    pvr->cmd_buffer[0] = 0x08;  /* write prefix */
    pvr->cmd_buffer[1] = 0x61;  /* i2c addr of tuner */
    pvr->cmd_buffer[2] = 0x04;  /* length of what follows */
    pvr->cmd_buffer[3] = val >> 8;
    pvr->cmd_buffer[4] = val & 0xff;
    pvr->cmd_buffer[5] = 0xce;
    pvr->cmd_buffer[6] = (pvr->frequency < 4800) ? 0x90 : 0x00 ;
    
    ret = pvr_send_request(pvr, pvr->cmd_buffer, 0x0b, pvr->cmd_buffer, 1);
    up(&pvr->ctl_sem);

    return ret;
}

/* read an u16 in  the eeprom. offset must be even. */

static int pvr_read_eeprom(struct pvr *pvr, u8 offset, u32 *res)
{
    int ret = 0;

//    pvr_debug("pvr_read_eeprom");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = 0x09;  /* read prefix */
    pvr->cmd_buffer[1] = 0x01;
    pvr->cmd_buffer[2] = 0x02;  /* length of answer. Device will send one more byte. */
    pvr->cmd_buffer[3] = 0x50;  /* i2c addr of eeprom */
    pvr->cmd_buffer[4] = offset;

    ret |= pvr_send_request(pvr, pvr->cmd_buffer, 0x0c, pvr->cmd_buffer, 3);
    *res = (pvr->cmd_buffer[1] << 8) + pvr->cmd_buffer[2];

    up(&pvr->ctl_sem);

    return ret;
}

static int pvr_write_encoder_cmd(struct pvr *pvr, u8 cmd, u32 data1,
                              u32 data2, u32 data3, u32 data4)
{
    int ret = 0;

    pvr_debug("pvr_write_encoder_cmd");

    down(&pvr->ctl_sem);

    memcpy(pvr->cmd_buffer, encoder_cmd_template.cmd, encoder_cmd_template.length);
    
    pvr->cmd_buffer[8] =  cmd;

    PVR_DECOMPOSE_LE(pvr->cmd_buffer,29,data1);
    PVR_DECOMPOSE_LE(pvr->cmd_buffer,36,data2);
    PVR_DECOMPOSE_LE(pvr->cmd_buffer,43,data3);
    PVR_DECOMPOSE_LE(pvr->cmd_buffer,50,data4);

    ret |= pvr_send_request(pvr, pvr->cmd_buffer,
                            encoder_cmd_template.length,
                            pvr->cmd_buffer,
                            encoder_cmd_template.ret);

    ret |= pvr_send_sequence(pvr, post_encoder_cmd);

    up(&pvr->ctl_sem);

    return ret;
}

static int pvr_write_decoder_cmd(struct pvr *pvr, u8 cmd, u8 data)
{
    int ret;

    pvr_debug("pvr_write_decoder_cmd");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = 0x08;  /* write prefix */
    pvr->cmd_buffer[1] = 0x21;  /* i2c addr of chip */
    pvr->cmd_buffer[2] = 0x02;  /* length of what follows */
    pvr->cmd_buffer[3] = cmd;
    pvr->cmd_buffer[4] = data;

    ret = pvr_send_request(pvr, pvr->cmd_buffer, 0x0b, pvr->cmd_buffer, 1);

    up(&pvr->ctl_sem);

    return ret;
}


static int pvr_write_audio_cmd(struct pvr *pvr, u8 dev, u16 addr, u16 data)
{
    int ret = 0;

    pvr_debug("pvr_write_audio_cmd");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = 0x08;  /* write prefix */
    pvr->cmd_buffer[1] = 0x40;  /* i2c addr of chip */
    pvr->cmd_buffer[2] = 0x05;  /* length of what follows */
    pvr->cmd_buffer[3] = dev;
    pvr->cmd_buffer[4] = (addr >> 8) &  0xff ;
    pvr->cmd_buffer[5] = addr &  0xff;
    pvr->cmd_buffer[6] = (data >> 8) &  0xff ;
    pvr->cmd_buffer[7] = data &  0xff;

    ret |= pvr_send_request(pvr, pvr->cmd_buffer, 0x0b, pvr->cmd_buffer, 1);

    up(&pvr->ctl_sem);

    return ret;
}

static int pvr_read_audio_cmd(struct pvr *pvr, u8 dev, u16 addr, u32 *res)
{
    int ret;

    pvr_debug("pvr_read_audio_cmd");

    down(&pvr->ctl_sem);

    pvr->cmd_buffer[0] = 0x09;  /* read prefix */
    pvr->cmd_buffer[1] = 0x03;  /* don't know what it is */
    pvr->cmd_buffer[2] = 0x02;  /* length of answer. Device will send one more byte. */
    pvr->cmd_buffer[3] = 0x40;  /* i2c addr of chip */
    pvr->cmd_buffer[4] = dev;
    pvr->cmd_buffer[5] = (addr >> 8) &  0xff;
    pvr->cmd_buffer[6] = addr &  0xff;

    ret = pvr_send_request(pvr, pvr->cmd_buffer, 0x0c, pvr->cmd_buffer, 3);
    *res = (pvr->cmd_buffer[1] << 8) + pvr->cmd_buffer[2];
    up(&pvr->ctl_sem);

    return ret;
}

/* 
 * Tables for automatic sound standard detection. Taken from msp3400.c from the 
 * ivtv projest (http://ivtv.sf.net). Some pieces of code for sound support
 * also come from there. Code is published  under GPL and Copyright is held by :
 * Gerd Knorr <kraxel@bytesex.org>, Pekka Pietikainen <pp@netppl.fi>,
 * Frederic Crozat <fcrozat@mail.dotcom.fr>
 *
 * Table might be WRONG...
 *
 */

static struct MODES {
	int retval;
	char *name;
} modelist[] = {
	{ 0x0000, "ERROR" },
	{ 0x0001, "autodetect start" },
	{ 0x0002, "4.5/4.72  M Dual FM-Stereo" },
	{ 0x0003, "5.5/5.74  B/G Dual FM-Stereo" },
	{ 0x0004, "6.5/6.25  D/K1 Dual FM-Stereo" },
	{ 0x0005, "6.5/6.74  D/K2 Dual FM-Stereo" },
	{ 0x0006, "6.5  D/K FM-Mono (HDEV3)" },
	{ 0x0008, "5.5/5.85  B/G NICAM FM" },
	{ 0x0009, "6.5/5.85  L NICAM AM" },
	{ 0x000a, "6.0/6.55  I NICAM FM" },
	{ 0x000b, "6.5/5.85  D/K NICAM FM" },
	{ 0x000c, "6.5/5.85  D/K NICAM FM (HDEV2)" },
	{ 0x0020, "4.5  M BTSC-Stereo" },
	{ 0x0021, "4.5  M BTSC-Mono + SAP" },
	{ 0x0030, "4.5  M EIA-J Japan Stereo" },
	{ 0x0040, "10.7  FM-Stereo Radio" },
	{ 0x0050, "6.5  SAT-Mono" },
	{ 0x0051, "7.02/7.20  SAT-Stereo" },
	{ 0x0060, "7.2  SAT ADR" },
	{     -1, NULL }, /* EOF */
};

int pvr_audio_setvolume(struct pvr *pvr)
{
    int bass, treble, volume = 0, balance = 0;
    int ret = 0;

    if (!pvr->audio_muted)
        volume = (pvr->audio_volume * 127 / 65535) << 8;
	
    if (pvr->audio_volume > 0)
        balance = (pvr->audio_balance * 127) / pvr->audio_volume;

	pvr_debug("setvolume: mute=%s v=0x%02x b=0x%02x",
		pvr->audio_muted ? "on" : "off", volume>>8, balance);
	
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0000, volume); /* loudspeaker */
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0006, volume); /* headphones  */
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0007,
                       pvr->audio_muted ? 0x1 : (volume | 0x1));
    /* scart - on/off only - AEW why? undone NOTE values below */
    /* 40000 are mostly useless, 59343 is a good default (0x73) */
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0001, balance << 8);

    bass = ((pvr->audio_bass - 32768) * 0x60 / 65535) << 8;
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0002, bass);

    treble = ((pvr->audio_treble - 32768) * 0x60 / 65535) << 8;
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0003, treble);

    return ret;
}

int pvr_configure_audio(struct pvr *pvr)
{
    int i, mode, std, ret = 0;
    int src = 0;
    u32 val;

    if (pvr->state <= _need_firmware)
        return 0;

    /* first reset the chip */
    ret |= pvr_do_command(pvr, &audio_reset_off, 0);
    ret |= pvr_do_command(pvr, &audio_reset_on, 0);

    std  = 1;

    switch (pvr->standard) {
        case PVR_STD_PAL:
            mode = 0x1003;
            break;
        case PVR_STD_NTSC:  /* BTSC */
            mode = 0x2003;
            std  = 0x0020;
            break;
        case PVR_STD_SECAM: 
            mode = 0x0003;
            break;
/*
		case PVR_MODE_RADIO: 
			mode = 0x0003;
			std  = 0x0040;
			break;
*/
        default:
            mode = 0x0003;
            break;
    }

    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DEM, 0x30, mode);

    /* set various prescales */
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0d, 0x3000); /* scart */
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0e, 0x2400); /* FM */
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x10, 0x5800); /* nicam */

    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DEM, 0x20, std);

    if (std != 1){
        /* programmed some specific mode */
        val = std;
    } else if (pvr->audio_input == PVR_INPUT_TV) {
        /* triggered autodetect */
        val = 0x07ff;
        i = 0;
        while((val >= 0x07ff) && (i < 20)){
            set_current_state(TASK_INTERRUPTIBLE);
            schedule_timeout(HZ / 10);
            ret |= pvr_read_audio_cmd(pvr, PVR_MSP3415_DEMR, 0x7e, &val);
            pvr_debug("audio autodetect read : 0x%08x", val);
            i++;
        }
    } else
        val = 0x07ff;

    for (i = 0; modelist[i].name != NULL; i++)
        if (modelist[i].retval == val)
            break;
    pvr_debug("setting audio mode: %s ",
			       modelist[i].name ? modelist[i].name : "unknown");

    /* FIXME : There is a workaround for AM in secam (?) */
        
    /* set stereo */
    switch (val) {
        case 0x0008:    /* B/G NICAM ? */
        case 0x000a:    /* I NICAM ? */
		case 0x0009:    /* AM mono 	? */	
            src = 0x0120;
            break;
		case 0x0020: /* BTSC ? */
			/* just turn on stereo */
            src = 0x0320;
			break;
		case 0x0040: /* FM radio ? */
			src = 0x0020;
			/* scart routing ? */
			break;
        case 0x07ff: /* RCA input */
            src = 0x0220;
            break;
    }

    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DEM, 0x0020, val);
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0008, src);
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0009, src);
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x000a, src);
    ret |= pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x000b, src);

    /* now select input through scart and set volume */
    pvr_write_audio_cmd(pvr, PVR_MSP3415_DFP, 0x0013,
                        (pvr->audio_input == PVR_INPUT_TV) ? 0x0f20 : 0x3760);
    pvr_audio_setvolume(pvr);

    return 0;
}

int pvr_decoder_set_input(struct pvr *pvr)
{
    switch(pvr->video_input){
        case PVR_INPUT_TV:
            return pvr_write_decoder_cmd(pvr, 0x02, 0x84);
        case PVR_INPUT_RCA:
            return pvr_write_decoder_cmd(pvr, 0x02, 0x85);
    }
    return -EINVAL;
}

int pvr_decoder_set_bcsh(struct pvr *pvr)
{
    pvr_write_decoder_cmd(pvr, 0x0a, pvr->video_brightness);
    pvr_write_decoder_cmd(pvr, 0x0b, pvr->video_contrast);
    pvr_write_decoder_cmd(pvr, 0x0c, pvr->video_saturation);
    pvr_write_decoder_cmd(pvr, 0x0d, pvr->video_hue);

    return -EINVAL;
}

static int pvr_configure_encoder(struct pvr *pvr)
{
    int ret = 0, audio;

    /* set stream output port. */
    ret |= pvr_write_encoder_cmd(pvr, 0xbb, 0x01, 0x01, 0, 0);
    
    /* set the Program Index Information. We want I,P,B frames (max 400) */
    ret |= pvr_write_encoder_cmd(pvr, 0xc7, 0x07, 0x0190, 0, 0);
    
    /* NOTE : windows driver sends some 0xdc */
    
    /* Strange compared to ivtv data. */
    ret |= pvr_write_encoder_cmd(pvr, 0xd6, 0x0120, 0x0120, 0, 0);
    
    /* setup firmware to notify us about some events (don't know why...) */
    ret |= pvr_write_encoder_cmd(pvr, 0xd5, 0, 0, 0x10000000, 0xffffffff);
    
    /* set fps to 25 or 30 (1 or 0)*/
    ret |= pvr_write_encoder_cmd(pvr, 0x8f, (pvr->standard != PVR_STD_NTSC),
                                0, 0, 0);
    /* set encoding resolution */
    ret |= pvr_write_encoder_cmd(pvr, 0x91, pvr->video_height,
                                pvr->video_width, 0, 0);
    /* set encoding aspect ratio to 4:3 */
    ret |= pvr_write_encoder_cmd(pvr, 0x99, 0x02, 0, 0, 0);
    
    /* disable VBI line information */
    ret |= pvr_write_encoder_cmd(pvr, 0xb7, 0xffffffff, 0, 0, 0);
    
    /* set stream type, depending on resolution. */
    ret |= pvr_write_encoder_cmd(pvr, 0xb9, (pvr->video_width == PVR_WIDTH_VCD) ?
                               0x0b : 0x0a, 0, 0, 0);
    /* set video bitrate */
    ret |= pvr_write_encoder_cmd(pvr, 0x95, pvr_tbl_vbr[pvr->video_vbr],
                                 pvr->video_bitrate,
                                 (int) pvr->video_peakbitrate / 400, 0);
    /* setup GOP structure (GOP size = 0f or 0c, 3-1 = 2 B-frames) */
    ret |= pvr_write_encoder_cmd(pvr, 0x97, (pvr->standard == PVR_STD_NTSC) ?
                                0x0f : 0x0c, 0x03, 0, 0);
    /* enable 3:2 pulldown */
    ret |= pvr_write_encoder_cmd(pvr, 0xb1, 0, 0, 0, 0);
    
    /* set GOP open/close property (open) */
    ret |= pvr_write_encoder_cmd(pvr, 0xc5, 0, 0, 0, 0);
    
    /* set audio stream properties */
    audio =   pvr_tbl_audiobitrate[pvr->audio_bitrate]
            | pvr_tbl_srate[pvr->audio_srate]
            | pvr->audio_layer << 2 
            | pvr_tbl_crc[pvr->audio_crc]
            | pvr_tbl_emphasis[pvr->audio_emphasis];

    pvr_debug("audio : %02x", audio);
    ret |= pvr_write_encoder_cmd(pvr, 0xbd, audio, 0, 0, 0);

    /* set dynamic noise reduction filter to manual, Horiz/Vert */
    ret |= pvr_write_encoder_cmd(pvr, 0x9b, 0, 0x03, 0, 0);
    
    /* dynamic noise reduction filter param */
    ret |= pvr_write_encoder_cmd(pvr, 0x9d, 0, 0, 0, 0);
    
    /* dynamic noise reduction median filter */
    ret |= pvr_write_encoder_cmd(pvr, 0x9f, 0, 0xff, 0, 0xff);
    
    /* spacial prefiler parameter */
    ret |= pvr_write_encoder_cmd(pvr, 0xa1, 0x01, 0x01, 0, 0);
    
    /* initialize video input */
    ret |= pvr_write_encoder_cmd(pvr, 0xcd, 0, 0, 0, 0);

    return ret;
}

static int pvr_configure_decoder(struct pvr *pvr)
{
    int ret = 0;

    /* Set some default values then set the right resolution. */
    
    ret |= pvr_do_command(pvr, &decoder_set_auto_input, 0);
    ret |= pvr_do_command(pvr, &decoder_set_misc1, 0);
    ret |= pvr_do_command(pvr, &decoder_set_misc2, 0);
    ret |= pvr_do_command(pvr, &decoder_set_fullres_x, 0);
    
    if (pvr->standard == PVR_STD_NTSC){
        ret |= pvr_do_command(pvr, &decoder_set_fullres_y_ntsc, 0);
        ret |= pvr_do_command(pvr, &decoder_set_video_ntsc, 0);
        switch(pvr->audio_srate){
            case PVR_SRATE_48:
                ret |= pvr_do_command(pvr, &decoder_set_audio_48_ntsc, 0);
                break;
            case PVR_SRATE_44:
                ret |= pvr_do_command(pvr, &decoder_set_audio_44_ntsc, 0);
                break;
        }
    } else {
        ret |= pvr_do_command(pvr, &decoder_set_fullres_y_pal, 0);
        ret |= pvr_do_command(pvr, &decoder_set_video_pal, 0);
        switch(pvr->audio_srate){
            case PVR_SRATE_48:
                ret |= pvr_do_command(pvr, &decoder_set_audio_48_pal, 0);
                break;
            case PVR_SRATE_44:
                ret |= pvr_do_command(pvr, &decoder_set_audio_44_pal, 0);
                break;
        }
    }

    if (pvr->video_width == PVR_WIDTH_VCD)
        ret |= pvr_do_command(pvr, &decoder_set_vcd_pal, 0);
    else if (pvr->video_width == PVR_WIDTH_SVCD)
        ret |= pvr_do_command(pvr, &decoder_set_svcd_pal, 0);

    ret |= pvr_do_command(pvr, &decoder_reset_scaler, 0);

    /* Don't know exactly  what this command does. Probably SECAM related ? */
    if (pvr->standard == PVR_STD_SECAM)
        ret |= pvr_do_command(pvr, &unk_cmd1, 0);
 
    return ret;
}

static int pvr_configure_capture(struct pvr *pvr)
{
    int ret = 0;

    ret |= pvr_configure_decoder(pvr);

    ret |= pvr_configure_encoder(pvr);

    ret |= pvr_set_freq(pvr);

    ret |= pvr_configure_audio(pvr);
    
    ret |= pvr_decoder_set_input(pvr);
    
    return ret;
}

/*
 * pvr_stop()
 *
 * Stop the stream. Call with lock held. After it, we can free all the buffer
 * without any danger, as long as we don't release the pvr->sem lock.
 */

static void pvr_stop(struct pvr *pvr)
{
    int i;

    pvr_debug("pvr_stop");

    for(i = 0 ; i < PVR_NBUFF; i++)
        usb_unlink_urb(pvr->buffer[i].purb);

    /* IS there is a small chance that an urb is back, but his completion
     * handler not yet complete ? 
     * We must wait a bit to be sure we have everybody...
     */

    if (wait_event_interruptible_timeout(pvr->urbs_collected,
    atomic_read(&pvr->running_urbs) == 0, HZ) == 0)
        err("timeout waiting for urbs !");

    /* mask all interrupts */
    pvr_write_register(pvr, 0x0048, 0xffffffff);
    /* stop capture on encoder side */
    pvr_write_encoder_cmd(pvr, 0x82, 0x01, 0, 0x13, 0);
    /* stop capture */
    pvr_write_u8(pvr, 0x37, 0);
    /* change some GPIO data */
    pvr_write_register(pvr, 0x9020, 0x81040000);
    pvr_write_register(pvr, 0x900c, 0x08000000);
}

/*
 * pvr_start()
 *
 * Start the stream. Call with lock held.
 *
 */

static void pvr_start(struct pvr *pvr)
{
    int i;

    pvr_debug("pvr_start");

    /* (re)initialize the rx ring. And submit urbs. */
    pvr->rx1 = 0;
    pvr->rx2 = 0;
    atomic_set(&pvr->running_urbs, PVR_NBUFF);
    for (i = 0 ; i < PVR_NBUFF ; i++)
        usb_submit_urb(pvr->buffer[i].purb, GFP_KERNEL);

    /* unmask some interrupts */
    pvr_write_register(pvr, 0x0048, 0xbfffffff);
    /* change some GPIO data */
    pvr_write_register(pvr, 0x9020, 0x80040000);
    pvr_write_register(pvr, 0x900c, 0x08000000);

    /* Start captutre (in MPEG) */
    pvr_do_command(pvr, &start_capture, 1);
    pvr_write_encoder_cmd(pvr, 0x81, 0, 0x13, 0, 0);

    pvr->init_timestamp = jiffies;
}

static void pvr_state_change(struct pvr *pvr, enum pvr_state state)
{
    if (pvr->state == state)
        return;

    if (pvr->state == _unstable)
        return;

    /* changing to a lower state */

    if (state == _unstable)
        goto done;   

    if ((state <= _stopped) && (pvr->state > _stopped))
        pvr_stop(pvr);

    if (state <= pvr->state)
        goto done;

    /* changing to an higher state */

    switch (pvr->state){
        case _need_firmware:
            if (state == _need_firmware)
                break;
            /* As it sometimes fails, give it 2 tries */
            if (pvr_upload_firmware2(pvr) && pvr_upload_firmware2(pvr)){
                err("device unstable!!");
                pvr->state = _unstable;
                return;
            }
        case _need_config:
            if (state == _need_config)
                break;
            pvr_configure_capture(pvr);
        case _stopped:
            if (state == _stopped)
                break;
            pvr_start(pvr);
        default:
            break;
    }

done:
    pvr->state = state;
    
}

static inline void pvr_state_above(struct pvr *pvr, enum pvr_state state)
{
    if (pvr->state < state)
        pvr_state_change(pvr, state);
}

static inline void pvr_state_below(struct pvr *pvr, enum pvr_state state)
{
    if (pvr->state > state)
        pvr_state_change(pvr, state);
}

inline void pvr_reconfigure(struct pvr *pvr)
{
    pvr_state_below(pvr, _need_config);
}

static int pvr_v4l_open(struct inode *, struct file *);
static int pvr_v4l_release(struct inode *, struct file *);
int pvr_v4l_do_ioctl(struct inode *, struct file *,
				 unsigned int, void *);
static int pvr_v4l_ioctl(struct inode *, struct file *,
		       unsigned int, unsigned long);
static ssize_t pvr_v4l_read(struct file *, char *, size_t,
                            loff_t *);
static unsigned int pvr_v4l_poll(struct file *, poll_table *);

static struct file_operations pvr_fops = {
	.owner		= THIS_MODULE,
	.open		= pvr_v4l_open,
	.release	= pvr_v4l_release,
	.read		= pvr_v4l_read,
	.ioctl		= pvr_v4l_ioctl,
	.llseek		= no_llseek,
    .poll       = pvr_v4l_poll,
};

static struct video_device pvr_video_template = {
	.owner		= THIS_MODULE,
	.type       = VFL_TYPE_GRABBER,
    .type2      = VID_TYPE_CAPTURE | VID_TYPE_TUNER,
    .hardware	= VID_HARDWARE_PVRUSB2,
	.fops		= &pvr_fops,
};

static void pvr_unregister_video(struct pvr *pvr)
{
	if (pvr->vdev) {
		info("unregister video /dev/video%d", pvr->vdev->minor & 0x1f);
		if (pvr->vdev->minor != -1)
			video_unregister_device(pvr->vdev);
		else
			video_device_release(pvr->vdev);
		pvr->vdev = NULL;
	}
}

/* register video4linux device */
static int __devinit pvr_register_video(struct pvr *pvr)
{
	struct video_device *vdev;

    if (pvr->udev == NULL) {
		err("%s: pvr->dev is not set", __FUNCTION__);
		goto exit;
	}

	vdev = video_device_alloc();
	if (vdev == NULL){
		err("%s: alloc of vdev failed", __FUNCTION__);;
        goto exit;
    }
	
	memcpy(vdev, &pvr_video_template, sizeof(struct video_device));
	pvr->vdev = vdev;
    vdev->dev = &pvr->udev->dev;
	vdev->release = video_device_release;
	video_set_drvdata(vdev, pvr);
    
	if (video_register_device(pvr->vdev, VFL_TYPE_GRABBER, video_nr))
		goto exit;

    info("pvrusb2: registered video device /dev/video%d",
				pvr->vdev->minor & 0x1f);
    
    return 0;

 exit:
	err("pvrusvb2: unable to register video device /dev/video%d", video_nr);
	pvr_unregister_video(pvr);
	return -1;
}


/*
 * pvr_open()
 *
 * This is part of Video 4 Linux API. The driver can be opened by multiple
 * clients (checks internal counter 'pvr->user'). This helps using v4lctl while
 * playing /dev/video?.
 * The procedure then allocates buffers needed for video processing.
 *
 */

static int pvr_v4l_open(struct inode *inode, struct file *file)
{
    struct video_device *vdev = video_devdata(file);
    struct pvr *pvr = (struct pvr *) video_get_drvdata(vdev);
    int ret = 0;

    pvr_debug("pvr_v4l_open");

    down(&pvr->sem);

    if (pvr->disconnecting){
        up(&pvr->sem);
        return -EIO;
    }

    if (pvr->user > 0){
        /* work has been done by previous users. */
        pvr->user++;
        up(&pvr->sem);
        return 0;
    }

    /* upload firmware encoder */
    pvr_state_above(pvr, _need_config);

    ret = pvr_alloc_video_buff(pvr);
    if (ret){
        err("alloc of video buffers failed");
        pvr_free_video_buff(pvr);
        up(&pvr->sem);
        return ret;
    }
    pvr->user++;

    up(&pvr->sem);

    return ret;
}

/*
 * pvr_release_final()
 *
 * This code does final release of struct pvr. This happens
 * after the device is disconnected and clients closed their files.
 *
 */
static void pvr_release_final(struct pvr *pvr)
{
	pvr_debug("pvr_final_release");

    if (pvr == NULL)
        return;

    pvr_unregister_video(pvr);

	kfree(pvr);
}

/*
 * pvr_v4l_release()
 *
 * This is part of Video 4 Linux API. The procedure
 * stops streaming and deallocates all buffers that were earlier
 * allocated in pvr_v4l_open().
 *
 */
static int pvr_v4l_release(struct inode *inode, struct file *file)
{
	struct video_device *vdev = video_devdata(file);
	struct pvr *pvr = (struct pvr *) video_get_drvdata(vdev);

    pvr_debug("pvr_v4l_release");

    down(&pvr->sem);

    pvr->user--;

    if(pvr->user > 0){
        /* work will be done  by someone else */
        up(&pvr->sem);
        return 0;
    }
    /* stop the stream... */
    pvr_state_below(pvr, _stopped);
    /* then free all the buffers */
    pvr_free_video_buff(pvr);
    up(&pvr->sem);

    if (pvr->disconnecting){
		pvr_debug("%s: Final disconnect", __FUNCTION__);
        pvr_release_final(pvr);
	}

    return 0;
}

static int pvr_v4l_ioctl(struct inode *inode, struct file *file,
		       unsigned int cmd, unsigned long arg)
{

/* Temporary hack : use ivtv api until a v4l2 one is available. */
#define IVTV_IOC_G_CODEC        0xFFEE7703
#define IVTV_IOC_S_CODEC        0xFFEE7704
    if (cmd == IVTV_IOC_G_CODEC || cmd == IVTV_IOC_S_CODEC)
        return 0;
    return video_usercopy(inode, file, cmd, arg, pvr_v4l_do_ioctl);
}

static ssize_t pvr_v4l_read(struct file *file, char *buff, size_t count,
                            loff_t *ppos)
{
    struct video_device *vdev = video_devdata(file);
    struct pvr *pvr = (struct pvr *) video_get_drvdata(vdev);
    struct pvr_buffer *b;
    int n;

//    pvr_debug("pvr_v4l_read : %i", count);

    if (!count) 
        return 0;

    down(&pvr->sem);

    if (!pvr->user || pvr->disconnecting || !pvr->udev){
        up(&pvr->sem);
        return -EIO;
    }

    /* FIXME : if (file->f_flags & O_NONBLOCK) is set, is it v4l2-legal
     * to do this ?
     */

    pvr_state_above(pvr, _should_stream);

    /* take data in the rx_buffer ring */
    if ((pvr->rx2 - pvr->rx1 == 0) && (file->f_flags & O_NONBLOCK)){
        /* prevent an infinite loop of such call */
        int ret = -EAGAIN;
        if ((pvr->state <= _should_stream) &&
            time_after(jiffies, pvr->init_timestamp + 2*HZ)){
            pvr_debug("nonblock init failed");
            pvr_state_below(pvr, _need_firmware);
            ret = -EIO;
        }
        up(&pvr->sem);
        return ret;
    }

    /* wait for some data to be available... */
    wait_event_interruptible_timeout(pvr->wait_data, pvr->rx2 - pvr->rx1 != 0 ,HZ);

    if (pvr->rx2 - pvr->rx1 == 0){
        pvr_debug("unable to get data (%i, %i) !!!!", atomic_read(&pvr->running_urbs), pvr->rx2);
        pvr_state_below(pvr, _need_firmware);
        up(&pvr->sem);
        return -EIO;
    }

    pvr_state_above(pvr, _streaming);    /* init was successfull*/
    b = pvr->rx_buffer[pvr->rx1 % PVR_NBUFF];
    if (b == NULL){
        err("ring is fucked !!!!");
        up(&pvr->sem);
        return -EIO;
    }
    n = min(b->size, count);
     
    copy_to_user(buff, b->data, n);
    if(n == b->size) {
        /* we took all the data in this urb. Re-submit it. */
        pvr->rx1++;
        atomic_inc(&pvr->running_urbs);
        if (!pvr->disconnecting)
            usb_submit_urb(b->purb, GFP_KERNEL);
    } else {
        /* Some data remains in this urb. Keep it. */
        b->data += n;
        b->size -= n;
    }

    up(&pvr->sem);

    return n;
}

static unsigned int pvr_v4l_poll(struct file *file, poll_table *wait)
{
    struct video_device *vdev = video_devdata(file);
    struct pvr *pvr = (struct pvr *) video_get_drvdata(vdev);
	unsigned int mask = 0;

    down(&pvr->sem);
    pvr_state_above(pvr, _should_stream);
    up(&pvr->sem);

	/* add stream's waitq to the poll list */
	poll_wait(file, &pvr->wait_data, wait);

	mask |= POLLIN | POLLRDNORM;

	return mask;
}

/*
 * pvr_probe()
 *
 * This procedure queries device descriptor and accepts the interface
 * if it is a WINTV-PVR USB2 video device.
 *
 */

static void pvr_configure_pvr(struct pvr *pvr)
{

#define PVR_DEFAULT_REG(id) (pvr_regular_ctl[id-V4L2_CID_BASE].default_value)
#define PVR_DEFAULT_PRIV(id) (pvr_private_ctl[id-V4L2_CID_PRIVATE_BASE].default_value)

    pvr->standard = PVR_STD_SECAM;
    pvr->video_input = 0;
    pvr->audio_input = 0;
    pvr->video_width = PVR_WIDTH_DVD;
    pvr->video_height = (pvr->standard == PVR_STD_NTSC) ? 480 : 576;
    pvr->frequency = 7796;
    pvr->video_bitrate = PVR_DEFAULT_PRIV(V4L2_CID_PVR_VIDEOBITRATE);
    pvr->video_vbr = PVR_DEFAULT_PRIV(V4L2_CID_PVR_VBR);
    pvr->video_peakbitrate = PVR_DEFAULT_PRIV(V4L2_CID_PVR_VIDEOPEAK);
    pvr->video_brightness = PVR_DEFAULT_REG(V4L2_CID_BRIGHTNESS);
    pvr->video_contrast = PVR_DEFAULT_REG(V4L2_CID_CONTRAST);
    pvr->video_saturation = PVR_DEFAULT_REG(V4L2_CID_SATURATION);
    pvr->video_hue = PVR_DEFAULT_REG(V4L2_CID_HUE);
    pvr->audio_bitrate = PVR_DEFAULT_PRIV(V4L2_CID_PVR_AUDIOBITRATE);
    pvr->audio_srate = PVR_DEFAULT_PRIV(V4L2_CID_PVR_SRATE);
    pvr->audio_crc = PVR_DEFAULT_PRIV(V4L2_CID_PVR_AUDIOCRC);
    pvr->audio_emphasis = PVR_DEFAULT_PRIV(V4L2_CID_PVR_AUDIOEMPHASIS);
    pvr->audio_layer = 2;
    pvr->audio_muted = PVR_DEFAULT_REG(V4L2_CID_AUDIO_MUTE);
    pvr->audio_volume = PVR_DEFAULT_REG(V4L2_CID_AUDIO_VOLUME);
    pvr->audio_balance = PVR_DEFAULT_REG(V4L2_CID_AUDIO_BALANCE);
    pvr->audio_bass = PVR_DEFAULT_REG(V4L2_CID_AUDIO_BASS);
    pvr->audio_treble = PVR_DEFAULT_REG(V4L2_CID_AUDIO_TREBLE);
}

static int pvr_probe(struct usb_interface *intf, const struct usb_device_id *devid)
{
	struct usb_device *udev = interface_to_usbdev(intf);
	__u8 ifnum = intf->cur_altsetting->desc.bInterfaceNumber;
	struct pvr *pvr = NULL;
    int ret;

	info("VID=%#04x, PID=%#04x, ifnum=%u",udev->descriptor.idVendor,
         udev->descriptor.idProduct, ifnum);

    info("endpoints : %i",intf->cur_altsetting->desc.bNumEndpoints);

    usb_set_interface(udev, ifnum, 0);

 	pvr = kmalloc(sizeof(struct pvr), GFP_KERNEL);
    if (pvr == NULL) {
		err("%s: couldn't allocate pvr struct", __FUNCTION__);
		return -ENOMEM;
	}

    memset(pvr, 0, sizeof(struct pvr));

    /*
     * If the FX2 firmware is not loaded, do it. Then the device will
     * re-enumerate, closing this driver instance.
     */
    if (intf->cur_altsetting->desc.bNumEndpoints == 0) {
        pvr->renum = 1;
        usb_set_intfdata(intf, pvr);
        pvr_upload_firmware1(udev, FIRMWARE1_FILE);
        return 0;
    }

    pvr->udev = udev;
    init_MUTEX(&pvr->sem);
    init_MUTEX(&pvr->ctl_sem);

    init_waitqueue_head (&pvr->wait_data);
    init_waitqueue_head (&pvr->urbs_collected);
    spin_lock_init (&pvr->ring_lock);

    pvr->nr = pvr_nr++;
    pvr->state = _need_firmware;
    pvr->user = 0;

    pvr_configure_pvr(pvr);

    ret = pvr_register_video(pvr);
    if (ret){
        kfree(pvr);
        return ret;
    }

    ret = pvr_input_init(pvr);
    if (ret){
        kfree(pvr);
        return ret;
    }
    usb_set_intfdata(intf, pvr);
    intf->minor = pvr->vdev->minor;
    pvr_input_start(pvr);

    return 0;
}

/*
 * pvr_disconnect()
 *
 */

static void pvr_disconnect(struct usb_interface *intf)
{
    struct pvr *pvr = usb_get_intfdata(intf);

    pvr_debug("pvr_disconnect");

    pvr->disconnecting = 1;

    if(pvr->renum){
        /* we did not initialize everything because device was 
         * going to re-enumerate
         */
        kfree(pvr);
        return;
    }

    down(&pvr->sem);

    /* Stop video io and remote stuff. */
    pvr_state_below(pvr, _stopped);
    pvr_input_stop(pvr);
    pvr_input_clear(pvr);

    usb_set_intfdata (intf, NULL);

    if (pvr->user) {
        pvr_debug("In use, disconnect pending");
        up(&pvr->sem);
    } else  {
        up(&pvr->sem); // needed ?
        pvr_release_final(pvr);
	}
}

static struct usb_driver pvr_driver = {
    owner:   THIS_MODULE,
    name:           "pvrusb2",
    id_table:       pvr_table,
    probe:          pvr_probe,
	disconnect:     pvr_disconnect
};

/*
 * pvr_init() / pvr_exit()
 *
 * This code is run to initialize/exit the driver.
 *
 */

static int __init pvr_init(void)
{
	int ret;

    pvr_debug("pvr_init");
 
    ret = usb_register(&pvr_driver);

	if (ret == 0)
		info(DRIVER_DESC " : " DRIVER_VERSION);

	return ret;
}

static void __exit pvr_exit(void)
{
    
    pvr_debug("pvr_exit");

    usb_deregister(&pvr_driver);
}

module_init(pvr_init);
module_exit(pvr_exit);

MODULE_DEVICE_TABLE (usb, pvr_table);
MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");
