/*
 *  
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <linux/version.h>
#include <linux/input.h>

#include "pvrusb2.h"

static char *name = "usb IR (pvrusb2)";
static char *phys = "usb"; // FIXME

#define PVR_KEYTAB_SIZE 0x40
#define PVR_KEYTAB_TYPE u32

PVR_KEYTAB_TYPE pvr_keycodes[PVR_KEYTAB_SIZE] = {
    [0x00] = KEY_KP0,
    [0x01] = KEY_KP1,
    [0x02] = KEY_KP2,
    [0x03] = KEY_KP3,
    [0x04] = KEY_KP4,
    [0x05] = KEY_KP5,
    [0x06] = KEY_KP6,
    [0x07] = KEY_KP7,
    [0x08] = KEY_KP8,
    [0x09] = KEY_KP9,
    [0x0b] = KEY_RED,
    [0x0c] = KEY_UNKNOWN,
    [0x0d] = KEY_MENU,
    [0x0f] = KEY_MUTE,
    [0x10] = KEY_VOLUMEUP,
    [0x11] = KEY_VOLUMEDOWN,
    [0x1e] = KEY_NEXT,
    [0x1f] = KEY_EXIT,
    [0x20] = KEY_CHANNELUP,
    [0x21] = KEY_CHANNELDOWN,
    [0x24] = KEY_PREVIOUS,
    [0x25] = KEY_OK,
    [0x29] = KEY_BLUE,
    [0x2e] = KEY_GREEN,
    [0x30] = KEY_PAUSE,
    [0x32] = KEY_REWIND,
    [0x34] = KEY_FORWARD,
    [0x35] = KEY_PLAY,
    [0x36] = KEY_STOP,
    [0x37] = KEY_RECORD,
    [0x38] = KEY_YELLOW,
    [0x3b] = KEY_GOTO,
    [0x3c] = KEY_SCREEN,
    [0x3d] = KEY_POWER
};

#define PVR_KEYCODE_MASK    0x00ff
#define PVR_KEYFLAG_MASK    0xff00
/* Each time a key is released this bit changes. This makes repeat
 * detection easy */
#define PVR_KEYFLAG_RE      0x02000

static void pvr_remote_task(struct pvr *pvr)
{
    int key, keycode, ret;

    ret = pvr_read_remote(pvr, &key);
    if (ret || ((key & PVR_KEYCODE_MASK) >> 2) >= PVR_KEYTAB_SIZE){
        pvr_debug("remote key reading failed");
        up(&pvr->task_sem);
        return;
    }
    if (pvr->last_key && (pvr->last_key != key)){
        pvr_debug("remote key released : 0x%04x", pvr->last_key);
        keycode = pvr_keycodes[(pvr->last_key & PVR_KEYCODE_MASK) >> 2];
        input_report_key(pvr->idev, keycode, 0);
        input_sync(pvr->idev);
    }
    if (key && (pvr->last_key != key)){
        pvr_debug("remote key pressed : 0x%04x", key);
        keycode = pvr_keycodes[(key & PVR_KEYCODE_MASK) >> 2];
        input_report_key(pvr->idev, keycode, 1);
        input_sync(pvr->idev);
    }

    pvr->last_key = key;
    up(&pvr->task_sem);
}


static void pvr_timer(unsigned long data)
{
    struct pvr *pvr = (struct pvr*) data;

    /* cannot do the work in interrupt context. */
    if (down_trylock(&pvr->task_sem) == 0)
        schedule_work(&pvr->remote_task);
    mod_timer(&pvr->remote_timer, jiffies + HZ / 4);
}


int pvr_input_init(struct pvr *pvr)
{
    int i;

    pvr_debug("pvr_input_init");
    pvr->idev = kmalloc(sizeof(struct input_dev), GFP_KERNEL);
    if (pvr->idev == NULL)
        return -ENOMEM;

    memset(pvr->idev, 0, sizeof(struct input_dev));

    init_input_dev(pvr->idev);
    pvr->idev->name = name;
    pvr->idev->id.bustype = BUS_USB;
    pvr->idev->phys = phys;

    pvr->idev->keycode = pvr_keycodes; 
    pvr->idev->keycodesize = sizeof(PVR_KEYTAB_TYPE);
    pvr->idev->keycodemax = PVR_KEYTAB_SIZE;

    for (i = 0; i < PVR_KEYTAB_SIZE; i++)
		set_bit(pvr_keycodes[i], pvr->idev->keybit);

    clear_bit(0, pvr->idev->keybit);

	set_bit(EV_KEY, pvr->idev->evbit);
	set_bit(EV_REP, pvr->idev->evbit); /* set repeat */

    input_register_device(pvr->idev);

    init_timer(&pvr->remote_timer);
    INIT_WORK(&pvr->remote_task, (void (*)(void *)) pvr_remote_task, pvr);
    init_MUTEX(&pvr->task_sem);

    return 0;
}

void pvr_input_start(struct pvr *pvr)
{
    pvr_debug("pvr_input_start");
    pvr->remote_timer.data = (unsigned long) pvr;
    pvr->remote_timer.function = &pvr_timer;
    mod_timer(&pvr->remote_timer, jiffies + 2 * HZ);
}

void pvr_input_stop(struct pvr *pvr)
{
    pvr_debug("pvr_input_stop");
    del_timer_sync(&pvr->remote_timer);
    /* flush scheduled work */
    down(&pvr->task_sem);
    up(&pvr->task_sem);
}

void pvr_input_clear(struct pvr *pvr)
{
    pvr_debug("pvr_input_clear");
    input_unregister_device(pvr->idev);
    kfree(pvr->idev);
}

