/*
 *  
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __PVRUSB2_H
#define __PVRUSB2_H


#include <linux/videodev.h>
#include <linux/usb.h>

/* size of a firmware chunk */
#define FIRMWARE_CHUNK_SIZE 0x2000

/* Various files we will load with firmware_entry */
#define FIRMWARE1_FILE  "pvrusb2.f1"
#define FIRMWARE2_FILE  "pvrusb2.f2"

/* Buffer management constants */
#define PVR_VID_BUFFSIZE        0x8000
#define PVR_CTL_BUFFSIZE        0x0100
#define PVR_NBUFF               6

#define PVR_VID_ENDPOINT        0x84
#define PVR_CTL_WRITE_ENDPOINT  0x01
#define PVR_CTL_READ_ENDPOINT   0x81
#define PVR_FIRMWARE_ENDPOINT   0x02

#define PVR_MSP3415_DEM   0x10
#define PVR_MSP3415_DEMR  0x11
#define PVR_MSP3415_DFP   0x12

#define PVR_DEBUG 1
#define pvr_debug(...)    do { if(PVR_DEBUG) info(__VA_ARGS__); } while(0)

#define VID_HARDWARE_PVRUSB2    38  /* FIXME : need a good value */

enum pvr_state {
    _unstable=0,    /* device is fucked up. Absorbant state. */
    _need_firmware, /* state 0 */
    _need_config,   /* encoder firmware in. Need to configure capture. */
    _stopped,       /* Device is ready to start capture. */
    _should_stream, /* we should deliver an MPEG stream */
    _streaming,     /* we deliver an MPEG stream */
};

struct pvr_cmd {
    u8 length; 
    u8 ret;
    u8 *cmd;
};

#define PVR_SIZEOf_CMD(c)   (2*sizeof(short)+sizeof(char)*(c)->length)

struct pvr_buffer {
    struct pvr *pvr;
    void *data;
    size_t size;
    struct urb *purb;
};

struct pvr {
    struct video_device *vdev;
    struct usb_device* udev;
    struct semaphore sem;
    int user;
    int nr;
    int disconnecting;
    int renum;
    unsigned long init_timestamp;
    enum pvr_state state;
    int reset;  /* to detect when a reset failed */
    
    atomic_t running_urbs; /* when trying to stop device */
    wait_queue_head_t urbs_collected;
    wait_queue_head_t wait_data;

    struct pvr_buffer buffer[PVR_NBUFF];       /* allocated buffers */
    struct pvr_buffer *rx_buffer[PVR_NBUFF];    /* ring of recv buffers */
    spinlock_t ring_lock;   /* we don't want simultaneous add to the ring. */
    unsigned int rx1;
    unsigned int rx2;

    /* our control pool */
    struct semaphore ctl_sem;
    unsigned char cmd_buffer[PVR_CTL_BUFFSIZE];  /* used under ctl_sem lock */
    
    /* remote stuff */
    struct input_dev* idev;
    struct timer_list remote_timer;    /* will read key pressed on the remote. */
    struct work_struct remote_task;
    struct semaphore task_sem;
    int last_key;
    
    /* audio/video parameters */
    unsigned int frequency;
    int video_input;
    int audio_input;
    int standard;
    int video_width;
    int video_height;
    int video_bitrate;
    int video_peakbitrate;
    int video_vbr;
    int video_brightness;
    int video_contrast;
    int video_saturation;
    int video_hue;
  
    int audio_bitrate;
    int audio_srate;
    int audio_layer;    /* only layer 2 seems supported */
    int audio_crc;
    int audio_emphasis;
    int audio_muted;
    int audio_volume;
    int audio_balance;
    int audio_bass;
    int audio_treble;
};

int pvr_configure_audio(struct pvr *);
int pvr_decoder_set_input(struct pvr *);
int pvr_decoder_set_bcsh(struct pvr *);
int pvr_encoder_set_audio(struct pvr *);
int pvr_audio_setvolume(struct pvr *);
int pvr_set_freq(struct pvr *);
int pvr_read_remote(struct pvr *, int *);
void pvr_reconfigure(struct pvr *);

#endif /* __PVRUSB2_H */
